@@ -1,26 +1,26 @@
-from __future__ import unicode_literals
-
+import copy
 import datetime
 from operator import attrgetter
-import os
-import pickle
-import subprocess
-import sys
-import unittest
 
 from django.core.exceptions import ValidationError
-from django.core.files.temp import NamedTemporaryFile
-from django.test import TestCase, skipUnlessDBFeature
-from django.utils import six
-from django.utils.timezone import get_fixed_timezone
-from django.db import connection, router
+from django.db import models, router
 from django.db.models.sql import InsertQuery
+from django.test import TestCase, skipUnlessDBFeature
+from django.test.utils import isolate_apps
+from django.utils.timezone import get_fixed_timezone
 
-from .models import (Worker, Article, Party, Event, Department,
-    BrokenUnicodeMethod, NonAutoPK, Model1, Model2, Model3)
+from .models import (
+    Article, Department, Event, Model1, Model2, Model3, NonAutoPK, Party,
+    Worker,
+)
 
 
 class ModelTests(TestCase):
+    def test_model_init_too_many_args(self):
+        msg = "Number of args exceeds number of fields"
+        with self.assertRaisesMessage(IndexError, msg):
+            Worker(1, 2, 3, 4)
+
     # The bug is that the following queries would raise:
     # "TypeError: Related Field has invalid lookup: gte"
     def test_related_gte_lookup(self):
@@ -54,10 +54,9 @@ class ModelTests(TestCase):
         # An empty choice field should return None for the display name.
         self.assertIs(a.get_status_display(), None)
 
-        # Empty strings should be returned as Unicode
+        # Empty strings should be returned as string
         a = Article.objects.get(pk=a.pk)
         self.assertEqual(a.misc_data, '')
-        self.assertIs(type(a.misc_data), six.text_type)
 
     def test_long_textfield(self):
         # TextFields can hold more than 4000 characters (this was broken in
@@ -148,11 +147,6 @@ class ModelTests(TestCase):
             attrgetter("when")
         )
 
-    if (3,) <= sys.version_info < (3, 3) and connection.vendor == 'mysql':
-        # In Python < 3.3, datetime.strftime raises an exception for years
-        # below 1000, and existing MySQL DB-API drivers hit this problem.
-        test_date_lookup = unittest.expectedFailure(test_date_lookup)
-
     def test_date_filter_null(self):
         # Date filtering was failing with NULL date values in SQLite
         # (regression test for #3501, among other things).
@@ -168,18 +162,12 @@ class ModelTests(TestCase):
         )
 
     def test_get_next_prev_by_field(self):
-        # Check that get_next_by_FIELD and get_previous_by_FIELD don't crash
-        # when we have usecs values stored on the database
-        #
-        # It crashed after the Field.get_db_prep_* refactor, because on most
-        # backends DateTimeFields supports usecs, but DateTimeField.to_python
-        # didn't recognize them. (Note that
-        # Model._get_next_or_previous_by_FIELD coerces values to strings)
+        # get_next_by_FIELD() and get_previous_by_FIELD() don't crash when
+        # microseconds values are stored in the database.
         Event.objects.create(when=datetime.datetime(2000, 1, 1, 16, 0, 0))
         Event.objects.create(when=datetime.datetime(2000, 1, 1, 6, 1, 1))
         Event.objects.create(when=datetime.datetime(2000, 1, 1, 13, 1, 1))
         e = Event.objects.create(when=datetime.datetime(2000, 1, 1, 12, 0, 20, 24))
-
         self.assertEqual(
             e.get_next_by_when().when, datetime.datetime(2000, 1, 1, 13, 1, 1)
         )
@@ -187,20 +175,22 @@ class ModelTests(TestCase):
             e.get_previous_by_when().when, datetime.datetime(2000, 1, 1, 6, 1, 1)
         )
 
+    def test_get_next_prev_by_field_unsaved(self):
+        msg = 'get_next/get_previous cannot be used on unsaved objects.'
+        with self.assertRaisesMessage(ValueError, msg):
+            Event().get_next_by_when()
+        with self.assertRaisesMessage(ValueError, msg):
+            Event().get_previous_by_when()
+
     def test_primary_key_foreign_key_types(self):
         # Check Department and Worker (non-default PK type)
         d = Department.objects.create(id=10, name="IT")
         w = Worker.objects.create(department=d, name="Full-time")
-        self.assertEqual(six.text_type(w), "Full-time")
-
-    def test_broken_unicode(self):
-        # Models with broken unicode methods should still have a printable repr
-        b = BrokenUnicodeMethod.objects.create(name="Jerry")
-        self.assertEqual(repr(b), "<BrokenUnicodeMethod: [Bad Unicode data]>")
+        self.assertEqual(str(w), "Full-time")
 
     @skipUnlessDBFeature("supports_timezones")
     def test_timezones(self):
-        # Saving an updating with timezone-aware datetime Python objects.
+        # Saving and updating with timezone-aware datetime Python objects.
         # Regression test for #10443.
         # The idea is that all these creations and saving should work without
         # crashing. It's not rocket science.
@@ -229,12 +219,30 @@ class ModelTests(TestCase):
         m3 = Model3.objects.get(model2=1000)
         m3.model2
 
+    @isolate_apps('model_regress')
+    def test_metaclass_can_access_attribute_dict(self):
+        """
+        Model metaclasses have access to the class attribute dict in
+        __init__() (#30254).
+        """
+        class HorseBase(models.base.ModelBase):
+            def __init__(cls, name, bases, attrs):
+                super().__init__(name, bases, attrs)
+                cls.horns = (1 if 'magic' in attrs else 0)
+
+        class Horse(models.Model, metaclass=HorseBase):
+            name = models.CharField(max_length=255)
+            magic = True
+
+        self.assertEqual(Horse.horns, 1)
+
 
 class ModelValidationTest(TestCase):
     def test_pk_validation(self):
         NonAutoPK.objects.create(name="one")
         again = NonAutoPK(name="one")
-        self.assertRaises(ValidationError, again.validate_unique)
+        with self.assertRaises(ValidationError):
+            again.validate_unique()
 
 
 class EvaluateMethodTest(TestCase):
@@ -244,48 +252,22 @@ class EvaluateMethodTest(TestCase):
 
     def test_model_with_evaluate_method(self):
         """
-        Ensures that you can filter by objects that have an 'evaluate' attr
+        You can filter by objects that have an 'evaluate' attr
         """
         dept = Department.objects.create(pk=1, name='abc')
         dept.evaluate = 'abc'
         Worker.objects.filter(department=dept)
 
 
-class ModelPickleTestCase(TestCase):
-    def test_unpickling_when_appregistrynotready(self):
-        """
-        #24007 -- Verifies that a pickled model can be unpickled without having
-        to manually setup the apps registry beforehand.
-        """
-        script_template = """#!/usr/bin/env python
-import pickle
+class ModelFieldsCacheTest(TestCase):
+    def test_fields_cache_reset_on_copy(self):
+        department1 = Department.objects.create(id=1, name='department1')
+        department2 = Department.objects.create(id=2, name='department2')
+        worker1 = Worker.objects.create(name='worker', department=department1)
+        worker2 = copy.copy(worker1)
 
-from django.conf import settings
-
-data = %r
-
-settings.configure(DEBUG=False, INSTALLED_APPS=('model_regress',), SECRET_KEY = "blah")
-article = pickle.loads(data)
-print(article.headline)"""
-        a = Article.objects.create(
-            headline="Some object",
-            pub_date=datetime.datetime.now(),
-            article_text="This is an article",
-        )
-
-        with NamedTemporaryFile(mode='w+', suffix=".py", dir='.') as script:
-            script.write(script_template % pickle.dumps(a))
-            script.flush()
-            try:
-                result = subprocess.check_output(
-                    [sys.executable, script.name],
-                    env={
-                        # Needed to run test outside of tests directory
-                        str('PYTHONPATH'): os.pathsep.join(sys.path),
-                        # Needed on Windows because http://bugs.python.org/issue8557
-                        str('PATH'): os.environ['PATH'],
-                    }
-                )
-            except subprocess.CalledProcessError:
-                self.fail("Unable to reload model pickled data")
-        self.assertEqual(result.strip().decode(), "Some object")
+        self.assertEqual(worker2.department, department1)
+        # Changing related fields doesn't mutate the base object.
+        worker2.department = department2
+        self.assertEqual(worker2.department, department2)
+        self.assertEqual(worker1.department, department1)
